#!/usr/bin/env python
"""
_ListWorkflow_

Oracle implementation of DBS3Buffer.ListWorkflow

Created on May 2, 2013

@author: dballest
"""

from WMComponent.DBS3Buffer.MySQL.ListWorkflow import ListWorkflow as MySQLListWorkflow

class ListWorkflow(MySQLListWorkflow):
    pass
