# -*- coding: utf-8 -*-
"""
_SetPendingJobSlotsForSite_

Oracle implementation of ResourceControl.SetPendingJobSlotsForSite

Created on Fri Jun 22 14:16:13 2012

@author: dballest
"""

from WMCore.ResourceControl.MySQL.SetPendingJobSlotsForSite import \
            SetPendingJobSlotsForSite as MySQLSetPendingJobSlotsForSite

class SetPendingJobSlotsForSite(MySQLSetPendingJobSlotsForSite):
    pass
